var Future = require("fibers/future");
var child_process = require("child_process");
var util = require("util");

var ChildProcess = (function () {
    function ChildProcess($logger) {
        this.$logger = $logger;
        this._exec = Future.wrap(function (command, callback) {
            return child_process.exec(command, callback);
        });
        this._execFile = Future.wrap(function (command, args, callback) {
            return child_process.execFile(command, args, callback);
        });
    }
    ChildProcess.prototype.exec = function (command) {
        this.$logger.debug("exec: %s", command);
        return this._exec(command);
    };

    ChildProcess.prototype.execFile = function (command, args) {
        var _this = this;
        this.$logger.debug("execFile: %s", command);
        args.forEach(function (a) {
            return _this.$logger.debug("    %s", a);
        });
        return this._execFile(command, args);
    };

    ChildProcess.prototype.spawn = function (command, args, options) {
        return child_process.spawn(command, args, options);
    };

    ChildProcess.prototype.spawnFromEvent = function (command, args, event, options) {
        var future = new Future();
        var childProcess = this.spawn(command, args, options);
        childProcess.once(event, function () {
            var args = _.toArray(arguments);
            var statusCode = args[0];

            if (statusCode !== 0) {
                future.throw(util.format("Command %s exited with code %s", command, statusCode));
            } else {
                future.return();
            }
        });

        return future;
    };
    return ChildProcess;
})();
exports.ChildProcess = ChildProcess;
$injector.register("childProcess", ChildProcess);
