"use strict";
var util = require("util");
var hostInfo = require("../../../common/host-info");
var MobileHelper = require("./../mobile-helper");

var IosEmulatorServices = (function () {
    function IosEmulatorServices($logger, $emulatorSettingsService, $errors, $childProcess) {
        this.$logger = $logger;
        this.$emulatorSettingsService = $emulatorSettingsService;
        this.$errors = $errors;
        this.$childProcess = $childProcess;
    }
    IosEmulatorServices.prototype.checkAvailability = function () {
        var _this = this;
        return (function () {
            if (!hostInfo.isDarwin()) {
                _this.$errors.fail("iOS Simulator is available only on Mac OS X.");
            }

            try  {
                _this.$childProcess.exec(util.format("which ", IosEmulatorServices.SimulatorLauncher)).wait();
            } catch (err) {
                _this.$errors.fail("Unable to find ios-sim. Run `npm install -g ios-sim` to install it.");
            }

            var platform = MobileHelper.DevicePlatforms[0 /* iOS */];
            if (!_this.$emulatorSettingsService.canStart(platform).wait()) {
                _this.$errors.fail("The current project does not target iOS and cannot be run in the iOS Simulator.");
            }
        }).future()();
    };

    IosEmulatorServices.prototype.startEmulator = function (app, emulatorOptions) {
        var _this = this;
        return (function () {
            _this.$logger.info("Starting iOS Simulator");
            var opts = [
                "launch", app,
                "--stderr", emulatorOptions.stderrFilePath,
                "--stdout", emulatorOptions.stdoutFilePath,
                "--family", emulatorOptions.deviceFamily
            ];
            _this.$childProcess.spawn(IosEmulatorServices.SimulatorLauncher, opts, { stdio: ["ignore", "ignore", "ignore"], detached: true }).unref();
        }).future()();
    };

    IosEmulatorServices.SimulatorLauncher = "ios-sim";
    return IosEmulatorServices;
})();
$injector.register("iOSEmulatorServices", IosEmulatorServices);
